


	;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
	;
	.MODULE	blaster

; all sprite routines assume 8:8 x, y values at (iy+OUSER)
;
BL_X			.equ	OUSER+0
BL_Y			.equ	OUSER+2
BL_FIRELATCH	.equ	OUSER+4
BL_MV			.equ	OUSER+5

blaster:
	; x = 28.0, y = 21.0
	;
	FPINIT(BL_X, 28, 0)
	FPINIT(BL_Y, 20, 0)

	ld		a,r
	and		7
	ld		(NEXTMTYPE),a

	ld		hl,_nextbltpos
	call	drawsprite

	; make sure 'base' (hahaha haven't used that name since 'invaders!)
	; is drawn 1st time through
	;
	inc		(iy+BL_MV)

_mainloop

_rdjoys:
	call	$1ffe
	ld		b,$ff
	rl		a
	rl		a
	rl		a			; bit 5 is now in carry
	jr		c,_tr
	res		3,b			; LEFT
_tr	
	rl		a			; bit 4 is now in carry
	jr		c,_tf
	res		4,b			; RIGHT
_tf	
	rl		a			; bit 3 is now in carry
	jr		c,_rdkeys
	res		7,b			; FIRE

_rdkeys:
	; look at the key row/column values as scanned during vsync.

	ld		a,(LAST_K)
	and		b			; clear any bits in A that we have stored in C
	ld		c,a

	ld		a,(iy+BL_X)

	; see if we should/can move... left
	;
	bit		3,c
	jr		nz,_testmoveright

isleft:
	; should move if we can.. if we're at the left edge don't do any more movement checks
	;
	cp		4
	jr		c,_testfire

	dec		a
	inc		(iy+BL_MV)
	jr		_setx

_testmoveright
	; see if we should/can move... right
	;
	bit		4,c
	jr		nz,_testfire

	cp		50
	jr		nc,_testfire

	inc		a
	inc		(iy+BL_MV)

_setx
	ld		(iy+BL_X),a

_testfire
	; either column in the bottom row can be used to fire, to suit left and right handers
	;
	ld		a,c
	and		%10000001
	cp		%10000001
	jr		nz,_couldbe

	; no key in the bottom row pressed - release the firebutton latch
	;
	ld		(iy+BL_FIRELATCH),0
	jr		_drawit

_couldbe
	; button down - only fire if the latch is clear..
	;
	bit		0,(iy+BL_FIRELATCH)
	jr		nz,_drawit

	; it's clear so set the firebutton latch
	;
	ld		(iy+BL_FIRELATCH),1

	; and launch a missile
	;
	call	getobject
	ld		bc,missile
	call	initobject
	call	insertobject_beforethis

	; new object's OUSER -> de
	;
	ld		d,h
	ld		e,l

	; transfer x & y
	;
	; **** assumes BL_X == OSUSER+0 && BL_Y == OUSER+2
	;
	ld		hl,(PSTORE)
	ldi
	ldi
	ldi
	ldi

	; transfer missile type
	;
	ld		a,(NEXTMTYPE)
	ld		(de),a

	call	RandLFSR
	and		7
	ld		(NEXTMTYPE),a

	ld		hl,_nextbltpos
	call	drawsprite

_drawit
	; all sprite routines assume x, y, type values at (iy+OUSER)
	;
	ld		a,(iy+BL_MV)
	or		a
	jr		z,_nodrawit

	ld		(iy+BL_MV),0
	ld		hl,(PSTORE)
	call	drawblaster

_nodrawit
	IDENT('B')
	YIELD

	ld		a,(TIMEOUT)
	or		a
	jr		nz,_gameover

	ld		a,(TARGETCOUNT)
	or		a
	jp		nz,_mainloop

_gameover
	ld		hl,(PSTORE)
	call	clearblaster	

	ld		a,8

	ld		hl,_nextbltpos
	call	drawsprite

	DIE

_nextbltpos
	db	25,0, 22,0


;------LFSR------
;James Montelongo
;optimized by Spencer Putt
;out:
; a = 8 bit random number
RandLFSR:
	push	hl
	push	de
	push	bc
	ld hl,LFSRSeed+4
	ld e,(hl)
	inc hl
	ld d,(hl)
	inc hl
	ld c,(hl)
	inc hl
	ld a,(hl)
	ld b,a
	rl e \ rl d
	rl c \ rla
	rl e \ rl d
	rl c \ rla
	rl e \ rl d
	rl c \ rla
	ld h,a
	rl e \ rl d
	rl c \ rla
	xor b
	rl e \ rl d
	xor h
	xor c
	xor d
	ld hl,LFSRSeed+6
	ld de,LFSRSeed+7
	ld bc,7
	lddr
	ld (de),a
	pop	bc
	pop	de
	pop	hl
	ret

LFSRSeed:
	.byte	0,$aa,0,$55

SeedRand:
	ld		a,(FRAMES)
	ld		(LFSRSeed),a
	ld		a,r
	ld		(LFSRSeed+2),a
	ret

	;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
	;
	.MODULE missile

ML_X		.equ	OUSER+0
ML_Y		.equ	OUSER+2
ML_TYPE		.equ	OUSER+4
ML_XSPEED	.equ	OUSER+5

missile:
	; adjust position of missile relative to blaster
	;
	inc		(iy+ML_X)
	inc		(iy+ML_X)
	inc		(iy+ML_X)
	inc		(iy+ML_X)
	dec		(iy+ML_Y)
	dec		(iy+ML_Y)

_shootloop
	; list the bullet in the... err.. bullet list :)
	;
	ld		hl,(NEXTBULLET)
	ld		(_lihi+1),hl
	inc		hl
	inc		hl
	ld		(NEXTBULLET),hl
	ld		hl,(PSTORE)
_lihi:
	ld		(0),hl

	; and up the active bullet count
	;
	ld		hl,BULLETCOUNT
	inc		(hl)

	; movin'on up
	;
	dec		(iy+ML_Y)

	ld		a,(iy+ML_TYPE)
	call	drawspriteiy

	ld		bc,64
	add		hl,bc
	CLEARHORIZONTAL3

	IDENT('B'+128)
	YIELD

	; upwards and onwards - unless we've hit the top line..!
	; 
	ld		a,(iy+ML_Y)
	cp		1
	jr		nz,_shootloop

	; bye bye
	
	ld		a,8
	call	drawspriteiy

	DIE


scoredhit:
	; calculate bonus multiplier
	;
	ld		b,0
	ld		a,(iy+ML_Y)

	; bonus of 1 if y == 11
	;
	cp		11
	jr		z,_is1

	; bonus of 2 if y > 6
	;
	cp		6
	jr		nc,_is2

_is3
	inc		b
_is2
	inc		b
_is1
	inc		b

	ld		a,b
	ld		(OSTORE+MA_BONEVAL),a
	ld		(OSTORE+MA_BONETRG),a

	; use a table here
	;
	ld		c,10
	ld		b,a
	xor		a

_multi
	add		a,c
	djnz	_multi

	ld		c,a
	
	ld		hl,(SCORE)
	add		hl,bc
	ld		(SCORE),hl
	
	ld		a,8
	call	drawspriteiy

	DIE



shootoff:
	ld		a,2
	ld		(tpenalty),a
	
	ld		a,r
	ld		(iy+ML_XSPEED),a
	bit		0,a
	jr		z,_shootoffleft

_shootoffright
	ld		a,(iy+ML_TYPE)
	call	drawspriteiy

	YIELD

	ld		a,8
	call	drawspriteiy

	ld		b,(iy+ML_XSPEED)
	FPADD(ML_X, 1, b)
	FPADD(ML_Y, 1, 0)


	ld		a,(iy+ML_Y)
	cp		20
	jr		z,_sodone
	ld		a,(iy+ML_X)
	cp		56
	jr		c,_shootoffright


_shootoffleft
	ld		a,(iy+ML_TYPE)
	call	drawspriteiy

	YIELD

	ld		a,8
	call	drawspriteiy

	ld		b,(iy+ML_XSPEED)
	FPSUB(ML_X, 1, b)
	FPADD(ML_Y, 1, 0)

	ld		a,(iy+ML_Y)
	cp		20
	jr		z,_sodone
	ld		a,(iy+ML_X)
	cp		2
	jr		nc,_shootoffleft

_sodone
	DIE
