;;
;; Compile with "tasm -80 -b main.asm swaparama.p"
;;

; TASM cross-assembler definitions
;
#define	db	.byte
#define	dw	.word
#define	ds	.block
#define	org	.org
#define	end	.end

; A useful reference
;
;  ____0___1___2___3___4___5___6___7___8___9___A___B___C___D___E___F____
;  00 SPC GRA GRA GRA GRA GRA GRA GRA GRA GRA GRA  "  GBP  $   :   ?  0F
;  10  (   )   >   <   =   +   -   *   /   ;   ,   .   0   1   2   3  1F
;  20  4   5   6   7   8   9   A   B   C   D   E   F   G   H   I   J  2F
;  30  K   L   M   N   O   P   Q   R   S   T   U   V   W   X   Y   Z  3F


; ostruct layout
;
ONEXT		.equ	$0
OPREV		.equ	$2
OPC			.equ	$4
OUSER		.equ	$6
;...
OSTRUCTSIZE	.equ	$40


; object store - we'll allocate space for 64 objects, this will occupy RAM
; from $6000-$7000
;
OSTORE		.equ	$6000
NSTRUCTS	.equ	$40


FREELIST	.equ	OSTORE+(NSTRUCTS*OSTRUCTSIZE)

BULLETLIST	.equ	FREELIST+(NSTRUCTS*2)

;
; process control macros
;

#define YIELD	call $+3\ pop de\ jp yield\
#define	DIE		push iy\ pop hl\ call unlinkobject\ jp resumenext

;
; debugging macros
;

;#define RESETINFOLINE  ld hl,(D_FILE)\ inc hl\ ld (INFO),hl\ ld b,32\ ld (hl),0\ inc hl\ djnz $-3
;#define IDENT(X) ld a,X-'A'+$26\ ld hl,(INFO)\ ld (hl),a\ inc hl\ ld (INFO),hl

#define RESETINFOLINE
#define IDENT(X)

;
; 8:8 fixed-point routines
;

; O is offset in private data area I is the int portion F is the fractional part.
;
#define FPINIT(O,I,F) ld (iy+O),I\ ld (iy+O+1),F

; exits with a = int portion of number
;
#define FPSUB(O,I,F) ld a,(iy+O+1)\ sub F\ ld (iy+O+1),a\ ld a,(iy+O)\ sbc a,I\ ld (iy+O),a
#define FPADD(O,I,F) ld a,(iy+O+1)\ add a,F\ ld (iy+O+1),a\ ld a,(iy+O)\ adc a,I\ ld (iy+O),a


	; -------------=-=-=-=-=-=-=-=-=-=-=-------------


			org	$4009

#include	"sysvars.asm"
#include	"zxRem0.asm"

	; -------------=-=-=-=-=-=-=-=-=-=-=-------------

PSTORE		.equ	PRTBUF+0		; pointer to private data store
BULLETCOUNT	.equ	PRTBUF+2		; number of bullets in the list
NEXTBULLET	.equ	PRTBUF+4		; pointer to where next bullet should be listed
TARGETCOUNT	.equ	PRTBUF+6		; number of targets for this level
ACTIVETGTCT	.equ	PRTBUF+8		; number of active targets
NEXTFREE	.equ	PRTBUF+10		; pointer to memory location holding address of next free object
SCORE		.equ	PRTBUF+12
NEXTMTYPE	.equ	PRTBUF+14		; next missile type
LEVEL		.equ	PRTBUF+16		; level
TIMERRESET	.equ	PRTBUF+18		; nonzero when timer in reset
TIMEOUT		.equ	PRTBUF+20		; nonzero when timer reaches 0

INFO		.equ	PRTBUF+28		; debugging helper

	; -------------=-=-=-=-=-=-=-=-=-=-=-------------

	.MODULE	gogamego

MA_BONETIME		.equ	OUSER+0
MA_BONEVAL		.equ	OUSER+1
MA_BONETRG		.equ	OUSER+2

gogamego:

;	#include "tests.asm"

	call	SeedRand
	ld		hl,0
	ld		(tpenalty),hl
	call	initostore

	; seed the random number generator
	;
	ld		a,r
	ld		(rng+1),a

	ld		de,(D_FILE)
	inc		de
	ld		hl,gamescreen
	call	rle
	call	aot

	; set up THIS (iy) pointer for the main object, as would have been done by the YIELD macro.
	;
	ld		iy,OSTORE

	; create the head of the linked object list: the 'main' object.
	; this waits for vsync, then proceeds along to the next object in the chain,
	; eventually returning to itself ad nauseum
	;
	; the head object is a special case - it will not be inserted into the chain -
	; we must set up the next pointer manually
	;
	call	getobject
	ld		bc,_fnmain
	call	initobject
	ld		hl,OSTORE
	ld		(OSTORE+ONEXT),hl
	ld		(OSTORE+OPREV),hl

	; the getready object fires off the 'levelator'
	; which in turn adds the targetgenerator, blaster and timer
	;
	call	resetgm

	;
	; get ready!
	;

	;
	; go!
	;

	ld		(iy+MA_BONETRG),0
	ld		(iy+MA_BONETIME),0

_fnmain
	call	wvs
	
	; draw the score
	;
	ld		hl,(D_FILE)
	ld		bc,23*33+2
	add		hl,bc
	ld		d,h
	ld		e,l
	ld		hl,(SCORE)
	call	decout

	; any bonus scored?
	;
	ld		a,(iy+MA_BONETRG)
	or		a
	jr		z,_nonewbonus

	; yes. clear the trigger, adjust the bonus type value,
	; and set up a timer
	;
	xor		a
	ld		(iy+MA_BONETRG),a
	dec		(iy+MA_BONEVAL)
	ld		a,50
	ld		(iy+MA_BONETIME),a

_nonewbonus
	ld		a,(iy+MA_BONETIME)
	or		a
	jr		z,_nobonus

	dec		a
	ld		(iy+MA_BONETIME),a
	ld		a,(iy+MA_BONEVAL)

_nobonus
	sla		a
	ld		c,a
	ld		b,0
	ld		hl,_bonustext
	add		hl,bc

	ex		de,hl
	
	ld		hl,(D_FILE)
	ld		bc,23*33+18
	add		hl,bc

	ex		de,hl
	ld		bc,2
	ldir

	;
	; get ready for next frame
	;

	; reset bullet list
	;
	xor		a
	ld		(BULLETCOUNT),a
	ld		hl,BULLETLIST
	ld		(NEXTBULLET),hl

	; do any debugging stuff
	;
	RESETINFOLINE

;	; show last_k value
;	ld		a,(LAST_K)
;	ld		hl,(D_FILE)
;	ld		de,16
;	add		hl,de
;	ex		de,hl
;	call	hexout

	
	; and mark the start of a wonderful new frame!
	;
	IDENT('M')
	YIELD

	jp		_fnmain


wvs:
	ld		hl,FRAMES
	ld		a,(hl)

_loop
	; wait for a vsync
	;
	cp		(hl)
	jr		z,_loop
	ret


resetgm:
	call	getobject
	ld		bc,getready
	call	initobject
	call	insertobject_afterhead

	ld		de,(D_FILE)
	inc		de
	ld		hl,gamescreen
	call	rle
	
	ld		hl,0
	ld		(SCORE),hl
	ld		(LEVEL),hl
	ld		(TIMERRESET),hl
	ld		(TIMEOUT),hl
	ret



_bonustext
	db	$00,$00
	db	$3d,$1e
	db	$3d,$1f
	
	; -------------=-=-=-=-=-=-=-=-=-=-=-------------

decout:
	ld		bc,-10000
	call	_digitout
	ld		bc,-1000
	call	_digitout
decout100:
	ld		bc,-100
	call	_digitout
decout10:
	ld		bc,-10
	call	_digitout
	ld		bc,-1

_digitout
	ld		a,$1c-1				; '0'-1
_doit
	inc		a
	add		hl,bc
	jr		c,_doit

	sbc		hl,bc

	ld		(de),a
	inc		de
	ret


hexout:
	ld		c,a
	srl		a
	srl		a
	srl		a
	srl		a
	add		a,$1c ; '0'
	ld		(de),a
	inc		de
	ld		a,c
	and		$f
	add		a,$1c ; '0'
	ld		(de),a
	ret

binout:
	ld		b,8
	ld		c,a
_boloop
	ld		a,$1c
	bit		7,c
	jr		z,_wr
	inc		a
_wr
	ld		(de),a
	inc		de
	sla		c
	djnz	_boloop
	ret


errorexit:
	jp		0



	.MODULE rng
rng:
	ld		a,0
	ld		b,a
	add		a,a
	add		a,a
	add		a,b
	inc		a
	ld		(rng+1),a
	ret
	
	
	; -------------=-=-=-=-=-=-=-=-=-=-=-------------

	#include "ostore.asm"
	#include "yield.asm"

	; -------------=-=-=-=-=-=-=-=-=-=-=-------------

	#include "sprite.asm"
	#include "generatd.asm"
	#include "rle.asm"

	; -------------=-=-=-=-=-=-=-=-=-=-=-------------
	
	#include "timer.asm"
	#include "blaster.asm"
	#include "target.asm"
	#include "present.asm"

	; -------------=-=-=-=-=-=-=-=-=-=-=-------------

;
;
;

line1:
	.byte	0,1   						; line number
	.word	xxdfile-$-2					; line length

	.byte	$f9,$d4,$c5					; RAND USR VAL
	.byte	$b,$1d,$22,$21,$1d,$20,$b	; "16514"
	.byte	$76    						; N/L

;- Display file --------------------------------------------

xxdfile:
	db	$76
	db	$76,$76,$76,$76,$76,$76,$76,$76
	db	$76,$76,$76,$76,$76,$76,$76,$76
	db	$76,$76,$76,$76,$76,$76,$76,$76

;- BASIC-Variables ----------------------------------------

var:
	db	$80

;- End of program area ----------------------------

last:

	end
