


;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
; table of pointers to functions that draw a sprite of a certain
; type at both even and odd boundaries. the odd sprite is shifted
; one med-res pixel right
;
srt:
	dw	backslash_e,backslash_o
	dw	fwdslash_e,fwdslash_o
	dw	topline_e,topline_o
	dw	bottomline_e,bottomline_o
	dw	leftline_e,leftline_o
	dw	rightline_e,rightline_o
	dw	allblack_e,allblack_o
	dw	clear_e,clear_o
	dw	eraser_e,eraser_o




;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;
; cleanup macros: all of which need hl to point to the proper
; place in the dfile.
;
#define CLEARHORIZONTAL2 ld (hl),0\ inc hl\ ld (hl),0
#define CLEARHORIZONTAL3 ld (hl),0\ inc hl\ ld (hl),0\ inc hl\ ld (hl),0
#define CLEARVERTICAL ld (hl),0\ ld bc,33\ add hl,bc\ ld (hl),0



	;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
	;
	.MODULE xy2dfile
	;
	; as above except the output value is dfile pointer
	;
	; [in] hl points to x,y
	; [out] hl: the pointer to the cell within d_file
	; [out] de: the pointer to the cell below within d_file
	; preserves: a
	;
xy2dfile:
	; x / 2
	;
	ld		c,(hl)
	srl		c

	; y
	;
	inc		hl
	inc		hl
	ld		l,(hl)

	ld		h,0
	ld		b,h

	; copy y for later
	;
	ld		d,h
	ld		e,l

	; y *= 32
	;
	sla		l
	rl		h
	sla		l
	rl		h
	sla		l
	rl		h
	sla		l
	rl		h
	sla		l
	rl		h

	; * 33
	;
	add		hl,de

	; hl = (y * 33) + x / 2
	;
	add		hl,bc

	; hl = screen cell
	;
	ld		de,(D_FILE)
	inc		de
	add		hl,de

	; de = hl
	;
	ld		d,h
	ld		e,l

	; hl = line below
	;
	ld		bc,33
	add		hl,bc

	; hl = top line, de = line below!
	;
	ex		de,hl

	ret



	;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
	;
	.MODULE drawsprite
	;
	; draws one of the compiled sprites.
	;
	; [in] hl: points to x,y
	; [in] a: contains the sprite number
	; [out] hl: points to last top line cell drawn
	; [out] de: points to last bottom line cell drawn
	; [preserves] nothing
	;
drawspriteiy:
	ld		hl,(PSTORE)

drawsprite:
	; de = a * 4 : offset to sprite routine address in table
	;
	sla		a
	sla		a
	ld		e,a
	ld		d,0

	bit		0,(hl)
	jr		z,_iseven

	; use odd sprite draw routine
	;
	inc		de
	inc		de

_iseven
	push	hl

	ld		hl,srt
	add		hl,de

	ld		de,_jumper+1		; prepare jump to sprite routine pointer
	ldi
	ldi

	pop		hl
	call	xy2dfile

	; jumps to the compiled sprite routine with:
	; hl = dfile address
	; de = dfile address + 33

_jumper
	jp		0					; jump to sprite routine pointer

	;
	;
	;

	.MODULE blastergfx

drawblaster:
	call	xy2dfile

	; make de = top line, save bot line..
	;
	ex		de,hl
	push	hl
	
	; assume even offset
	;
	ld		hl,blaster_e

	; if x is odd..
	;
	bit		0,(iy+OUSER)
	jr		z,_goblaster

	; draw odd blaster
	;
	ld		hl,blaster_o

_goblaster
	ld		bc,6
	ldir
	pop		de
	ld		bc,6
	ldir
	ret

	;-=-=-=-=-=-=-=-=
	;

clearblaster:
	call	xy2dfile
	ld		b,6
	ld		a,131

_cbloop
	ld		(hl),0
	ld		(de),a
	inc		hl
	inc		de
	djnz	_cbloop

	ret


blaster_e:
	db		0,135,128,128,4,0
	db		131,129,128,128,130,131

blaster_o:
	db		0,0,129,128,130,0
	db		131,131,128,128,128,131
