


	;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
	;
	.MODULE	targetgenerator

TARGETDLY		.equ	100
TARGETSPD		.equ	70

TG_GENREMAINCT	.equ	OUSER+0
TG_GENDLY		.equ	OUSER+1
TG_GENTYPE		.equ	OUSER+2
TG_BASEDLY		.equ	OUSER+3
TG_BASESPD		.equ	OUSER+4

targetgenerator:
	; these are modified when a generator is created
	;
	ld		a,TARGETDLY
	ld		b,TARGETSPD

	ld		(iy+TG_BASEDLY),a
	ld		(iy+TG_BASESPD),b

	ld		a,0
	ld		(ACTIVETGTCT),a

	ld		a,10
	ld		(TARGETCOUNT),a
	ld		(iy+TG_GENREMAINCT),a		; number to generate

	ld		a,(iy+TG_BASEDLY)			; frames to wait before generating a target
	ld		(iy+TG_GENDLY),a

	ld		(iy+TG_GENTYPE),-1			; sprite type [0..6]

_generator
	; is it time to generate?
	;
	dec		(iy+TG_GENDLY)
	jr		nz,_nogen

	; reset delay
	;
	ld		a,(iy+TG_BASEDLY)
	ld		(iy+TG_GENDLY),a 

	; reduce remaining count
	;
	dec		(iy+TG_GENREMAINCT)

	; and generate a target
	;
	call	getobject
	ld		bc,target
	call	initobject
	call	insertobject_afterthis

	; determine target type
	;
	ld		a,(iy+TG_GENTYPE)
	inc		a
	and		7
	ld		(iy+TG_GENTYPE),a

	; hl is new object's pstore
	;
	ld		(hl),a
	inc		hl
	ld		a,(iy+TG_BASESPD)
	ld		(hl),a

_nogen
	IDENT('G')
	YIELD

	; probably the game is wrong if the timeout signal fires before
	; all the targets are generated...
	ld		a,(TIMEOUT)
	or		a
	jr		nz,_gameover

	; any left to generate
	;
	ld		a,(IY+TG_GENREMAINCT)
	or		a
	jr		nz,_generator

	; now we're in the post-generation phase - watch for the last tgt to go,
	; then start the 'next level' object

_targetleft
	IDENT('G'+128)
	YIELD

	ld		a,(TIMEOUT)
	or		a
	jr		nz,_gameover
	
	; any left running?
	;
	ld		a,(TARGETCOUNT)
	or		a
	jr		nz,_targetleft

	; kill the timer
	;
	inc		a
	ld		(TIMERRESET),a

	call	getobject
	ld		bc,getready
	call	initobject
	call	insertobject_afterhead

_gameover
	DIE






	;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
	;
	.MODULE target

TGT_X			.equ	OUSER+0
TGT_Y			.equ	OUSER+2
TGT_TYPE		.equ	OUSER+4
TGT_SPEED		.equ	OUSER+5

target:
	; move the type and speed
	;
	ld		a,(iy+OUSER)
	ld		(iy+TGT_TYPE),a

	ld		a,(iy+OUSER+1)
	ld		(iy+TGT_SPEED),a

	ld		hl,ACTIVETGTCT
	inc		(hl)
	
	; x,y are stored as 8:8 fixed point
	;
_targetreset
	FPINIT(TGT_X, 60, 0)
	FPINIT(TGT_Y, 3, 0)

	; move right to left
	;
	; each phase takes a copy of the int part of the coordinate that's
	; being updated and only redraws a sprite when that coord changes.
	;
	ld		c,(iy+TGT_X)

_phase0
	ld		b,(iy+TGT_SPEED)
	FPSUB(TGT_X, 0, b)
	cp		c
	jr		z,_phase0_nodraw

	ld		a,(iy+TGT_TYPE)
	call	drawspriteiy

	; on return, hl will point to the last char in the top line
	; of the sprite that was drawn. carry is set if it's an odd X
	;
	jr		c,_phase0_nodraw
	inc		hl
	CLEARVERTICAL

_phase0_nodraw
	IDENT('T')
	YIELD

	ld		a,(TIMEOUT)
	or		a
	jp		nz,_generalexit

	call	getcollision
	jp		c,_collision

	; move on to next phase after we've drawn ourselves at final X
	;
	ld		a,(iy+TGT_X)
	ld		c,a
	cp		2
	jr		nz,_phase0

	; -=-=-=-=-=-=-=-=-=-=-
	; drop 3 lines
	;
	ld		c,(iy+TGT_Y)

_phase1
	ld		b,(iy+TGT_SPEED)
	FPADD(TGT_Y, 0, b)
	cp		c
	jr		z,_phase1_nodraw

	ld		a,(iy+TGT_TYPE)
	call	drawspriteiy
	
	ld		bc,-33-1
	add		hl,bc
	CLEARHORIZONTAL2

_phase1_nodraw
	IDENT('T')
	YIELD

	ld		a,(TIMEOUT)
	or		a
	jp		nz,_generalexit

	call	getcollision
	jp		c,_collision

	ld		a,(iy+TGT_Y)
	ld		c,a
	cp		7
	jr		nz,_phase1

	; -=-=-=-=-=-=-=-=-=-=-
	; move left to right
	;
	ld		c,(iy+TGT_X)

_phase2
	ld		b,(iy+TGT_SPEED)
	FPADD(TGT_X, 0, b)
	cp		c
	jr		z,_phase2_nodraw

	ld		a,(iy+TGT_TYPE)
	call	drawspriteiy
	jr		c,_phase2_nodraw
	
	dec		hl
	dec		hl
	CLEARVERTICAL

_phase2_nodraw
	IDENT('T')
	YIELD

	ld		a,(TIMEOUT)
	or		a
	jp		nz,_generalexit

	call	getcollision
	jp		c,_collision

	ld		a,(iy+TGT_X)
	ld		c,a
	cp		58
	jr		nz,_phase2

	; -=-=-=-=-=-=-=-=-=-=-
	; drop 3 lines
	;
	ld		c,(iy+TGT_Y)

_phase3
	ld		b,(iy+TGT_SPEED)
	FPADD(TGT_Y, 0, b)
	cp		c
	jr		z,_phase3_nodraw

	ld		a,(iy+TGT_TYPE)
	call	drawspriteiy

	ld		bc,-33-1
	add		hl,bc
	CLEARHORIZONTAL2

_phase3_nodraw
	IDENT('T')
	YIELD

	ld		a,(TIMEOUT)
	or		a
	jr		nz,_generalexit

	call	getcollision
	jr		c,_collision

	ld		a,(iy+TGT_Y)
	ld		c,a
	cp		11
	jr		nz,_phase3

	; -=-=-=-=-=-=-=-=-=-=-
	; move right to left
	;
	ld		c,(iy+TGT_X)

_phase4
	ld		b,(iy+TGT_SPEED)
	FPSUB(TGT_X, 0, b)
	cp		c
	jr		z,_phase4_nodraw

	ld		a,(iy+TGT_TYPE)
	call	drawspriteiy
	jr		c,_phase4_nodraw

	inc		hl
	CLEARVERTICAL

_phase4_nodraw
	IDENT('T')
	YIELD

	ld		a,(TIMEOUT)
	or		a
	jr		nz,_generalexit

	call	getcollision
	jr		c,_collision

	ld		a,(iy+TGT_X)
	ld		c,a
	or		a
	jr		nz,_phase4

	and		a					; clear carry

_collision
	; reset/collision! carry flag indicates a collision happened
	;
	push	af
	push	hl

	ld		a,8
	call	drawspriteiy

	pop		hl
	pop		af
	jp		nc,_targetreset

	; there was a collision, so do what needs to be done...
	; inform the missile that it's collided by altering its resume address.. cheeky!

	; get missile type
	;
	inc		hl					; ->Y.f
	inc		hl					; ->type
	ld		a,(hl)

	; add instead of sbc
	;
	ld		bc,-((OUSER+2)-OPC+2)
	add		hl,bc

	cp		(iy+TGT_TYPE)
	jr		z,_heshootshescores

	; we hit, but the types are incompatible..
	;
	ld		(hl),shootoff % 256
	inc		hl
	ld		(hl),shootoff / 256
	jr		_generalexit

_heshootshescores
	ld		(hl),scoredhit % 256
	inc		hl
	ld		(hl),scoredhit / 256

_generalexit
	ld		hl,TARGETCOUNT
	dec		(hl)
	ld		hl,ACTIVETGTCT
	dec		(hl)

	DIE



	;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
	;
	.MODULE	getcollision

temphl		dw	0
tempbc		dw	0



getcollision:
	; no bullets active? nothing to do!
	;
	ld		a,(BULLETCOUNT)
	or		a					; clears carry which is handy
	ret		z

	; object x,y into de
	;
	ld		d,(iy+TGT_X)
	ld		e,(iy+TGT_Y)

	ld		hl,BULLETLIST
	ld		b,a

_nexttest
	ld		(tempbc),bc
	ld		(temphl),hl

	; bullet -> hl
	;
	ld		a,(hl)
	inc		hl
	ld		h,(hl)
	ld		l,a

	; bullet x:y in bc
	;
	ld		b,(hl)
	inc		hl
	inc		hl
	ld		c,(hl)
	
	; no collision if abs(dx) > 3 (we're using double-res horizontally)
	;
	ld		a,b
	sub		d

	; if there is a collision, we expect a value in the range -3..3.
	; add in a value that will set the least expected value to 0. in this
	; case 3. the range is now 0..6 inclusive. subtracting rangemax+1 (6+1)
	; will result in a carry if the tested value is in the range.
	;
	add		a,3
	sub		7
	jr		c,_nextcolchk

	; hmm i'm not using the stack in order to make some logic below simpler.
	; i'm not quite sure about it yet. TODO: find another way.
	;
	ld		hl,(temphl)
	ld		bc,(tempbc)
	inc		hl
	inc		hl

	djnz	_nexttest
	and		a					; clear carry
	ret

_nextcolchk
	; it's looking more likely - the 1st test passed. no collision if abs(dy) > 2
	; IOW if a is in the range -1,0,1
	;
	ld		a,c
	sub		e
	inc		a
	cp		2
	ret		c

	ld		hl,(temphl)
	ld		bc,(tempbc)
	inc		hl
	inc		hl

	djnz	_nexttest
	and		a					; clear carry
	ret
